from typing import Any, Dict
from urllib.parse import quote_plus, urlencode

import requests

from fastel.exceptions import APIException
from fastel.social import SocialModel


class FacebookLogin:
    def __init__(self, data: SocialModel) -> None:
        self.project_client_id = data.project_client_id
        self.project_api_host = data.project_api_host
        self.client_id = data.client_id
        self.client_secret = data.client_secret

        self.CALLBACK_URL = f"{self.project_api_host}/facebook/callback"
        self.FACEBOOK_AUTH_DIALOG = f"https://www.facebook.com/v9.0/dialog/oauth?client_id={self.client_id}&redirect_uri={self.CALLBACK_URL}&scope=email,public_profile"
        self.FACEBOOK_URL = "https://graph.facebook.com"

    def redirect(self, state: str) -> str:
        url = self.FACEBOOK_AUTH_DIALOG + f"&state={state}"
        return url

    def callback(self, state: Dict[str, Any], code: str) -> Dict[str, Any]:
        payload = {
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "redirect_uri": self.CALLBACK_URL,
            "code": code,
        }
        path = urlencode(query=payload, quote_via=quote_plus)  # type: ignore
        resp = requests.get(self.FACEBOOK_URL + "/v9.0/oauth/access_token?" + path)
        resp_json = resp.json()

        try:
            resp.raise_for_status()
        except Exception:
            print(resp_json)
            raise APIException(
                status_code=401,
                error="callback_error",
                detail=resp_json["detail"],
            )

        access_token = resp_json["access_token"]
        profile_resp = requests.get(
            self.FACEBOOK_URL
            + f"/v9.0/me?fields={state.get('fields', 'name,email,id')}&access_token={access_token}"
        )
        profile = profile_resp.json()
        result = {}
        result["social_id"] = profile["id"]
        result["provider"] = "facebook"

        try:
            result["email"] = profile["email"]
            result["name"] = profile["name"]
        except Exception:
            print(
                f"[WARNING] fb social user {result['social_id']} doesn't have email or name"
            )

        return result



from functools import wraps
from typing import Any, Callable, TypeVar, cast

import requests as _requests

FuncT = TypeVar("FuncT", bound=Callable[..., Any])
WrapT = TypeVar("WrapT", bound=Callable[..., Any])


def log_all(action: str) -> WrapT:
    def inner(fn: FuncT) -> FuncT:
        @wraps(fn)
        def wrapper(*args: Any, **kwargs: Any) -> Any:
            url: str = args[0]
            json = kwargs.get("json", None)
            print("[REQ]", action, url, json)
            return fn(*args, **kwargs)

        return cast(FuncT, wrapper)

    return cast(WrapT, inner)


@log_all("get")  # type: ignore
def get(*args: Any, **kwargs: Any) -> Any:
    return _requests.get(*args, **kwargs)


@log_all("post")  # type: ignore
def post(*args: Any, **kwargs: Any) -> Any:
    return _requests.post(*args, **kwargs)


@log_all("put")  # type: ignore
def put(*args: Any, **kwargs: Any) -> Any:
    return _requests.put(*args, **kwargs)


@log_all("delete")  # type: ignore
def delete(*args: Any, **kwargs: Any) -> Any:
    return _requests.delete(*args, **kwargs)



from typing import Any, Dict, Tuple

import google_auth_oauthlib.flow
import requests

from fastel.exceptions import APIException
from fastel.social import SocialModel


class GoogleLogin:
    def __init__(self, data: SocialModel) -> None:
        self.project_api_host = data.project_api_host
        self.redirect_uri = f"{data.project_api_host}/google/callback"
        self.flow = google_auth_oauthlib.flow.Flow.from_client_config(
            {
                "web": {
                    "client_id": data.client_id,
                    "project_id": data.client_id,
                    "auth_uri": "https://accounts.google.com/o/oauth2/auth",
                    "token_uri": "https://oauth2.googleapis.com/token",
                    "auth_provider_x509_cert_url": "https://www.googleapis.com/oauth2/v1/certs",
                    "client_secret": data.client_secret,
                    "redirect_uris": [self.redirect_uri],
                }
            },
            scopes=[
                "openid",
                "https://www.googleapis.com/auth/userinfo.email",
                "https://www.googleapis.com/auth/userinfo.profile",
            ],
        )
        self.flow.redirect_uri = self.redirect_uri

    def redirect(self) -> Tuple[str, str]:
        authorization_url, state = self.flow.authorization_url(
            include_granted_scopes="true"
        )
        return authorization_url, state

    def callback(
        self, state: Dict[str, Any], full_query_param_str: str
    ) -> Dict[str, Any]:
        self.flow.state = state["state"]
        self.flow.fetch_token(
            authorization_response=f"{self.redirect_uri}?{full_query_param_str}"
        )
        access_token = self.flow.credentials.token

        resp = requests.get(
            f"https://people.googleapis.com/v1/people/me?personFields=names,emailAddresses&access_token={access_token}"
        )
        profile = resp.json()

        try:
            resp.raise_for_status()
        except Exception:
            raise APIException(
                status_code=401,
                error="http_error_occur",
                detail=profile["detail"],
            )

        result = {}
        result["social_id"] = profile["names"][0]["metadata"]["source"]["id"]
        result["provider"] = "google"

        try:
            result["email"] = profile["emailAddresses"][0]["value"]
            result["name"] = profile["names"][0]["unstructuredName"]
        except Exception:
            print(
                f"[WARNING] google social user {result['social_id']} doesn't have email or name"
            )

        return result



import random
from typing import Any, Type

from .decode_static import prod_cert, stg_cert
from .jwt_payload import JWTPayloadBuilder
from .kms_encode import kms_encode as encode


class JWTEncoder:
    payload_builder_class: Type[JWTPayloadBuilder]
    stage: str = "stg"

    def __init__(self, client: Any) -> None:
        self.client = client

    def encode(self, user: Any) -> str:
        keys = stg_cert if self.stage == "stg" else prod_cert
        key = random.choice(keys["keys"])["kid"]
        builder = self.payload_builder_class(client=self.client, data=user)
        payload = builder.get_payload()
        encoded = encode(payload, key=key, headers={"kid": key})
        return encoded
